 //this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#pragma once
#include "afxdlgs.h"
#include "ResizableLib\ResizableDialog.h"
#include "OScopeCtrl.h"
#include "otherfunctions.h"
#include "ColorFrameCtrl.h"
#include "COPTIONTREE\OptionTree.h"
#include <list>
#include "clientlist.h"
#include "WebServer.h"
using namespace std;

extern bool g_bUseNewStyle;

class CStatisticsData
{
public:
	virtual ~CStatisticsData();
	void AddEntry(unsigned int nID, CString sTitle, unsigned int nLevel);
	void SetTitle(unsigned int nID, CString sTitle);
	void SetStatic(unsigned int nID, CString sStatic);
	void SetValue(unsigned int nID, CString sValue, bool bReverse = false);
	void Expand(unsigned int nID, bool bExpand);
	bool IsExpanded(unsigned int nID);
	HWND GetListHWND();
	void SetRedraw(BOOL bRedraw);
	void SetColumn(unsigned int nWidth){ m_ctrlStatsList.SetColumn(nWidth); }
	unsigned int GetColumn(){ return m_ctrlStatsList.GetColumn(); }
	void CreateNewStyle(CWnd *pParent, CRect &rect)
	{
		m_ctrlStatsList.Create(WS_VISIBLE | WS_CHILD | WS_BORDER | WS_TABSTOP |
			LVS_REPORT | LVS_ALIGNLEFT | LVS_SINGLESEL,
			rect, pParent, OT_OPTIONS_SHADEEXPANDCOLUMN | OT_OPTIONS_SHADEROOTITEMS, rand());
	}
	void ExpandAllItems();
	CString ExportHTML();

	CTreeCtrl	stattree;

private:
	class CDataEntry
	{
	public:
		unsigned int			m_nID;
		CString					m_sTitle;
		CString					m_sValue;
		unsigned int			m_nLevel;
		COptionTreeItem*		m_pTopItem;
		COptionTreeItemStatic*	m_pSubItem;
		CDataEntry*				m_pParent;
		HTREEITEM				m_hItem;

		CDataEntry(unsigned int nID = 0, CString sTitle = _T(""), CDataEntry* pParent = NULL);
		COptionTreeItem* GetItem();
		CDataEntry* GetParent(unsigned int nParentLevel);
		void Expand(bool bExpand);
		bool IsExpanded();
	};

	CDataEntry* FindItem(unsigned int nID);

	CArray<CDataEntry*, CDataEntry*> m_DataList;

	COptionTree	m_ctrlStatsList;
};

// CStatisticsDlg dialog

class CStatisticsDlg : public CResizableDialog
{
	DECLARE_DYNAMIC(CStatisticsDlg)

	friend class CWebServer;

public:
	CStatisticsDlg(CWnd* pParent = NULL);   // standard constructor
	~CStatisticsDlg();
	enum { IDD = IDD_STATISTICS };

	void 	Localize();
	void 	SetCurrentRate(float uploadrate, float downloadrate);
	void 	ShowInterval();
	void 	ShowStatistics();
	void 	UpdateActConScale();
		
	void 	SetARange(bool SetDownload,int maxValue);
	void 	RecordRate();
	float 	GetAvgDownloadRate(int averageType);
	float 	GetAvgUploadRate(int averageType);
	void 	RepaintMeters();
	void UpdateConnectionsStatus();
	float GetMaxConperFiveModifier();
	CString GetStatsListExpanded();
	long 	GetStatsColumnWidth()
	{
		return g_bUseNewStyle ? m_Stats.GetColumn() : 150;
	}

private:
	bool m_bFirstShow;

    COScopeCtrl m_DownloadOMeter,m_UploadOMeter,m_Statistics;
	CColorFrameCtrl m_Led1[ 3 ];
	CColorFrameCtrl m_Led2[ 3 ];
	CColorFrameCtrl m_Led3[ 4 ];

	void CreateStatsList();

	int 	dl_users,dl_active;
	double 	m_dPlotDataMore[4];
	int 	stat_max;
	float 	maxDownavg;
	float 	maxDown;
	uint32 	m_ilastMaxConnReached;
	uint8	oldRatio;

	void 	SetupLegend( int ResIdx, int ElmtIdx, int legendNr);
	void 	SetStatsRanges(int min, int max);

	const 	static int AVG_SESSION =0;
	const 	static int AVG_TIME =1;
	list<uint64> uprateHistory;
	list<uint64> downrateHistory;
	list<DWORD>timeHistory;
    uint32	peakconnections;
	uint32	totalconnectionchecks;
	float	averageconnections;
	uint32	activeconnections;
	bool	bInited;
protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support	
	DECLARE_MESSAGE_MAP()
	virtual BOOL OnInitDialog(); 
	void	OnShowWindow(BOOL bShow,UINT nStatus);
public:
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnDestroy();	// eklmn: bugfix(00): resource cleanup due to CResizableDialog
private:
	CStatisticsData	m_Stats;
};
